# Generated by Django 2.0.4 on 2018-10-15 03:35
import os
from logging import warning
from shutil import copyfile

from django.conf import settings
from django.db import migrations


SPECT_FFT_TEMPLATE = os.path.join(settings.MEDIA_URL, 'spect', 'syllable', '{}.png')
SPECT_FFT_BAK_TEMPLATE = os.path.join(settings.MEDIA_URL, 'spect', 'syllable', '{}-bak.png')


def change_spectrogram_to_use_tid(apps, schema_editor):
    """
    Up to now the spects are stored using syllable IDs. When a syllable is copied, a new and identical spect is made.
    This is wasteful & slowing down the process of copying significantly.
    This commit will change the storage to use TID, which is the same for original and the copy, and only changes
    when the copy changes.
    This migration remove all identical copies of the original syllable
    """
    db_alias = schema_editor.connection.alias
    model = apps.get_model('koe', 'Segment')

    vl = model.objects.using(db_alias).all().values_list('id', 'tid')
    tids = model.objects.using(db_alias).all().values_list('tid', flat=True)
    tid2ids = {x: [] for x in tids}
    to_delete = []

    for id, tid in vl:
        tid2ids[tid].append(id)
    for tid, ids in tid2ids.items():

        # The original syllable is always the one with smallest ID
        min_id = min(ids)
        for id in ids:
            sid_path = SPECT_FFT_TEMPLATE.format(id)
            if id == min_id:
                tid_path_bak = SPECT_FFT_BAK_TEMPLATE.format(tid)
                if os.path.isfile(sid_path):
                    copyfile(sid_path, tid_path_bak)
                else:
                    warning('File {} is missing'.format(sid_path))
            else:
                if os.path.isfile(sid_path):
                    to_delete.append(sid_path)

    for sid_path in to_delete:
        os.remove(sid_path)

    for tid, ids in tid2ids.items():
        min_id = min(ids)
        for id in ids:
            if id == min_id:
                tid_path = SPECT_FFT_TEMPLATE.format(tid)
                tid_path_bak = SPECT_FFT_BAK_TEMPLATE.format(tid)
                if os.path.isfile(tid_path_bak):
                    os.rename(tid_path_bak, tid_path)


class Migration(migrations.Migration):
    dependencies = [
        ('koe', '0013_auto_20181015_0335'),
    ]

    operations = [
        migrations.RunPython(change_spectrogram_to_use_tid, reverse_code=migrations.RunPython.noop),
    ]
