# Generated by Django 2.0.4 on 2020-03-14 08:11

import django.db.models.deletion
from django.db import migrations
from django.db import models
from django.db import transaction
from django.db.models import Q
from progress.bar import Bar


def add_individual_and_track(apps, schema_editor):
    """
    """
    db_alias = schema_editor.connection.alias
    AudioFile = apps.get_model('koe', 'AudioFile')
    Individual = apps.get_model('koe', 'Individual')
    AudioTrack = apps.get_model('koe', 'AudioTrack')

    afs = AudioFile.objects.using(db_alias).filter(original=None).filter(Q(individual=None) | Q(track=None))
    bar = Bar('Setting placeholder for original AudioFile\'s individual and track', max=len(afs))

    with transaction.atomic():
        for af in afs:
            if af.individual is None:
                individual = Individual.objects.get_or_create(name=af.id)[0]
                af.individual = individual

            if af.track is None:
                track = AudioTrack.objects.get_or_create(name=af.id)[0]
                af.track = track

            af.save()
            bar.next()
        bar.finish()

    afs = AudioFile.objects.using(db_alias).exclude(original=None).filter(Q(individual=None) | Q(track=None))
    bar = Bar('Setting placeholder for copied AudioFile\'s individual and track', max=len(afs))

    with transaction.atomic():
        for af in afs:
            if af.individual is None:
                af.individual = af.original.individual

            if af.track is None:
                af.track = af.original.track

            af.save()
            bar.next()
        bar.finish()


class Migration(migrations.Migration):

    dependencies = [
        ('koe', '0031_rearrange_spectrograms_by_page'),
    ]

    operations = [
        migrations.RunPython(add_individual_and_track, reverse_code=migrations.RunPython.noop),
        migrations.AlterField(
            model_name='audiofile',
            name='individual',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='koe.Individual'),
        ),
        migrations.AlterField(
            model_name='audiofile',
            name='track',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='koe.AudioTrack'),
        ),
    ]
